cordova.define("cordova-plugin-device.device", function(require, exports, module) {  /*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
*/

var argscheck = require('cordova/argscheck'),
    channel = require('cordova/channel'),
    utils = require('cordova/utils'),
    exec = require('cordova/exec'),
    cordova = require('cordova');

channel.createSticky('onCordovaInfoReady');
// Tell cordova channel to wait on the CordovaInfoReady event
channel.waitForInitialization('onCordovaInfoReady');

/**
 * This represents the mobile device, and provides properties for inspecting the model, version, UUID of the
 * phone, etc.
 * @constructor
 */
function Device() {
    this.available = false;
    this.platform = null;
    this.version = null;
    this.uuid = null;
    this.cordova = null;
    this.model = null;
    this.manufacturer = null;

    var me = this;

    channel.onCordovaReady.subscribe(function() {
        me.getInfo(function(info) {
            //ignoring info.cordova returning from native, we should use value from cordova.version defined in cordova.js
            //TODO: CB-5105 native implementations should not return info.cordova
            var buildLabel = cordova.version;
            me.available = true;
            me.platform = info.platform;
            me.version = info.version;
            me.uuid = info.uuid;
            me.cordova = buildLabel;
            me.model = info.model;
            me.manufacturer = info.manufacturer || 'unknown';
            me.domain = info.domain;
            me.mallDomain = info.mallDomain;
            me.userImgDomain = info.userImgDomain;
            me.api_name = info.api_name;
            me.api_key = info.api_key;
            me.api_token = info.api_token;
            me.debugMode = info.debugMode;
            me.cacheMode = info.cacheMode;
            me.projectName = info.projectName;
            me.rootuser = info.rootuser;
            me.mallRootUser = info.mallRootUser;
            me.dal = info.dal;
            me.addr_userlogin = info.addr_userlogin;
            me.addr_userCheck = info.addr_userCheck;
            me.addr_userRegister = info.addr_userRegister;
            me.addr_devicelist = info.addr_devicelist;
            me.addr_gdevicelist = info.addr_gdevicelist;
            me.addr_addibleproductorList = info.addr_addibleproductorList;
            me.addr_checkMacAddress = info.addr_checkMacAddress;
            me.addr_userinfo = info.addr_userinfo;
            me.addr_userLoginOut = info.addr_userLoginOut;
            me.addr_updateInfoForWifiReset = info.addr_updateInfoForWifiReset;
            me.addr_updateUserInfo = info.addr_updateUserInfo;
            me.addr_submitFeedback = info.addr_submitFeedback;
            me.addr_aboutUs = info.addr_aboutUs;
            me.addr_deviceAuthority = info.addr_deviceAuthority;
            me.add_deleteSecondary = info.add_deleteSecondary;
            me.addr_gdeviceerrorlist = info.addr_gdeviceerrorlist;
            me.addr_gphonenumber = info.addr_gphonenumber;
            me.addr_deleteDeviceErrorInfo = info.addr_deleteDeviceErrorInfo;
            me.addr_updateDeviceErrorInfo = info.addr_updateDeviceErrorInfo;
            me.add_deviceLog = info.add_deviceLog;
            me.addr_gdevicesetting = info.addr_gdevicesetting;
            me.addr_updateDeviceSetting = info.addr_updateDeviceSetting;
            me.addr_compareVersion = info.addr_compareVersion;
            me.addr_forgerPassSend = info.addr_forgerPassSend;
            me.addr_regPassSend = info.addr_regPassSend;
            me.addr_resetPassword = info.addr_resetPassword;
            me.addr_gdevicefans = info.addr_gdevicefans;
            me.addr_deleteDeviceId = info.addr_deleteDeviceId;
            me.addr_setDeviceTop = info.addr_setDeviceTop;
            me.addr_deleteFollowDeviceId = info.addr_deleteFollowDeviceId;
            me.addr_nearbybranch = info.addr_nearbybranch;
            me.addr_getUserMessage = info.addr_getUserMessage;
            me.addr_gattentdevicelist = info.addr_gattentdevicelist;
            me.addr_getnearbydevice = info.addr_getnearbydevice;
            me.addr_gnickdevicelist = info.addr_gnickdevicelist;
            me.addr_checkIfBinded = info.addr_checkIfBinded;
            me.addr_getmessage = info.addr_getmessage;
            me.addr_updateDevice = info.addr_updateDevice;
            me.addr_getProductInfo = info.addr_getProductInfo;
            me.addr_firmwareUpdateFlg = info.addr_firmwareUpdateFlg;
            me.addr_getMySetting = info.addr_getMySetting;
            me.addr_updateMySetting = info.addr_updateMySetting;
            me.addr_getDeviceCard = info.addr_getDeviceCard;
            me.addr_deletemessage = info.addr_deletemessage;
            me.addr_updatemessage = info.addr_updatemessage;
            me.addr_gettiming = info.addr_gettiming;
            me.addr_settiming = info.addr_settiming;
            me.addr_thirdPartyLoginIn = info.addr_thirdPartyLoginIn;
            me.addr_thirdUserBindFlyco = info.addr_thirdUserBindFlyco;
            me.addr_accountUnbind = info.addr_accountUnbind;
            me.addr_addDeviceCardWatchLog = info.addr_addDeviceCardWatchLog;
            me.addr_flycoBindThirdParty = info.addr_flycoBindThirdParty;
            me.addr_deviceCardControl = info.addr_deviceCardControl;
            me.addr_deviceCardConcern = info.addr_deviceCardConcern;
            me.addr_getDeviceDtl = info.addr_getDeviceDtl;
            me.addr_getHintsDetail = info.addr_getHintsDetail;
            me.addr_updateFirmware = info.addr_updateFirmware;
            me.addr_flycoUnbindThirdUser = info.addr_flycoUnbindThirdUser;
            me.addr_shareBarCode = info.addr_shareBarCode;
            me.mall_codeLogin = info.mall_codeLogin;
            me.mall_integraln = info.mall_integraln;
            me.mall_codeDelete = info.mall_codeDelete;
            me.mall_mallindex = info.mall_mallindex;
            me.mall_groupshopping = info.mall_groupshopping;
            me.mall_paytype = info.mall_paytype;
            me.mall_addresslist = info.mall_addresslist;
            me.mall_orderlist = info.mall_orderlist;
            me.mall_logisticsdetail = info.mall_logisticsdetail;
            me.mall_orderdetail = info.mall_orderdetail;
            me.mall_newProduct = info.mall_newProduct;
            me.mall_secKill = info.mall_secKill;
            me.mall_goodsCategory = info.mall_goodsCategory;
            me.mall_goodsList = info.mall_goodsList;
            me.mall_goodsDetail = info.mall_goodsDetail;
            me.mall_refundapply = info.mall_refundapply;
            me.mall_returnapply = info.mall_returnapply;
            me.mall_orderListReturn = info.mall_orderListReturn;
            me.mall_orderListRefund = info.mall_orderListRefund;
            me.mall_getCartList = info.mall_getCartList;
            me.mall_getRefundList = info.mall_getRefundList;
            me.mall_getAddressList = info.mall_getAddressList;
            me.mall_editAddress = info.mall_editAddress;
            me.mall_addAddress = info.mall_addAddress;
            me.mall_delAddress = info.mall_delAddress;
            me.mall_getRegionList = info.mall_getRegionList;
            me.mall_myFavouriteList = info.mall_myFavouriteList;
            me.mall_delFavourite = info.mall_delFavourite;
            me.mall_region = info.mall_region;
            me.mall_goodsSimpleInfo = info.mall_goodsSimpleInfo;
            me.mall_setDefaultAddress = info.mall_setDefaultAddress;
            me.mall_getMyFavouriteList = info.mall_getMyFavouriteList;
            me.mall_delMyFavourite = info.mall_delMyFavourite;
            me.mall_addMyFavourite = info.mall_addMyFavourite;
            me.mall_getGoodsComment = info.mall_getGoodsComment;
            me.mall_getGoodsNocomment = info.mall_getGoodsNocomment;
            me.mall_addComment = info.mall_addComment;
            me.mall_getIntegralList = info.mall_getIntegralList;
            me.mall_getIntegralList2 = info.mall_getIntegralList2;
            me.mall_getCheckInInfo = info.mall_getCheckInInfo;
            me.mall_doCheckIn = info.mall_doCheckIn;
            me.mall_getDefaultAddress = info.mall_getDefaultAddress;
            me.mall_goodsComment = info.mall_goodsComment;
            me.mall_cancelOrder = info.mall_cancelOrder;
            me.mall_addOrder = info.mall_addOrder;
            me.mall_personHot = info.mall_personHot;
            me.mall_confirmRecept = info.mall_confirmRecept;
            me.mall_getPromo = info.mall_getPromo;
            me.mall_goodsSearch = info.mall_goodsSearch;
            me.mall_addToMyCart = info.mall_addToMyCart;
            me.mall_mycart = info.mall_mycart;
            me.mall_deleteGoodsInMyCart = info.mall_deleteGoodsInMyCart;
            me.mall_cleanMyCart = info.mall_cleanMyCart;
            me.mall_baoYouLine = info.mall_baoYouLine;
            me.mall_goodsCollect = info.mall_goodsCollect;
            me.mall_payStatus = info.mall_payStatus;
            me.mall_payPrepare = info.mall_payPrepare;
            me.mall_uploadImage = info.mall_uploadImage;
            me.mall_editCart = info.mall_editCart;
            me.mall_sessionIdToUserId = info.mall_sessionIdToUserId;
            me.mall_suitCommodity = info.mall_suitCommodity;
            me.mall_signDesc = info.mall_signDesc;
            me.mall_goodSalesNumber = info.mall_goodSalesNumber;
            me.app_version = info.app_version;
            channel.onCordovaInfoReady.fire();
        },function(e) {
            me.available = false;
            utils.alert("[ERROR] Error initializing Cordova: " + e);
        });
    });
}

/**
 * Get device info
 *
 * @param {Function} successCallback The function to call when the heading data is available
 * @param {Function} errorCallback The function to call when there is an error getting the heading data. (OPTIONAL)
 */
Device.prototype.getInfo = function(successCallback, errorCallback) {
    argscheck.checkArgs('fF', 'Device.getInfo', arguments);
    exec(successCallback, errorCallback, "Device", "getDeviceInfo", []);
};

module.exports = new Device();

});
